/**************************************************************************
 *   Copyright (C) 2018 by Thomas Sailer                                   *
 *   t.sailer@alumni.ethz.ch                                               *
 *                                                                         *
 *   Copyright (C) 2012 by Andreas Fritiofson                              *
 *   andreas.fritiofson@gmail.com                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef OPENOCD_JTAG_DRIVERS_MPSSE_INTERNAL_H
#define OPENOCD_JTAG_DRIVERS_MPSSE_INTERNAL_H

#include "mpsse.h"

#define BITMODE_RESET 0x00
#define BITMODE_MPSSE 0x02

struct mpsse_backend_func {
	void (*close)(struct mpsse_ctx *ctx);
	int (*flush)(struct mpsse_ctx *ctx);
	void (*purge)(struct mpsse_ctx *ctx);	
};

struct mpsse_ctx {
	const struct mpsse_backend_func *backend;
	enum ftdi_chip_type type;
	uint8_t *write_buffer;
	unsigned write_size;
	unsigned write_count;
	uint8_t *read_buffer;
	unsigned read_size;
	unsigned read_count;
	struct bit_copy_queue read_queue;
	int retval;
};

/* Buffer handling */
unsigned mpsse_buffer_write_space(struct mpsse_ctx *ctx);
unsigned mpsse_buffer_read_space(struct mpsse_ctx *ctx);
void mpsse_buffer_write_byte(struct mpsse_ctx *ctx, uint8_t data);
unsigned mpsse_buffer_write(struct mpsse_ctx *ctx, const uint8_t *out, unsigned out_offset,
	unsigned bit_count);
unsigned mpsse_buffer_add_read(struct mpsse_ctx *ctx, uint8_t *in, unsigned in_offset,
	unsigned bit_count, unsigned offset);

/* Device handling */
struct mpsse_ctx *mpsse_libusb1_open(const uint16_t *vid, const uint16_t *pid, const char *description,
	const char *serial, const char *location, int channel);
struct mpsse_ctx *mpsse_d2xx_open(const uint16_t *vid, const uint16_t *pid, const char *description,
	const char *serial, const char *location, int channel);

#endif /* OPENOCD_JTAG_DRIVERS_MPSSE_INTERNAL_H */
